// =========================================================================================================
//  Mine Field
//  Version: 1.0.0
//  Author: Kronzky (www.kronzky.info / kronzky@gmail.com)
// ---------------------------------------------------------------------------------------------------------
//
// Fills a marker-defined area with mines
//
// To create mines at random positions within the marker area, only the marker name is needed.
// In that case anti-personnel mines are created, using the default values as documented below.
//   nil=["areamarker"] execVM "minefield.sqf"
//
// ---------------------------------------------------------------------------------------------------------
//
// Required parameter:
//      "areamarker" : Name of marker that designates the area to be covered.
//                     The marker has to be a rectangle, and can be rotated.
//
// Optional parameters:
// (default values)
//
//      triggerclass : The "lightest" type of unit that should activate the trigger.
//      ("Land")        Typically, this would be something like "Land", "Man", "LandVehicle" or "Tank".
//                     A complete list of vehicle types is available at the Wiki:
//                       http://community.bistudio.com/wiki/ArmA:_CfgVehicles 
// 
//      quantity     : How many mines should be created.
//      (1 per 10 square meters)
//
//      range        : How close a unit has to be to activate the trigger.
//      (.75)
//
//      "explosion"  : What type of ammo to use to create the mine explosion.
//      ("G_40mm_HE")  A list of available ammunition types is available at the Wiki: 
//                       http://community.bistudio.com/wiki/ArmA:_Weapons#Vehicle_Weapons (use the name from the ammo column)
//                     Keep in mind that the damage depends on the type of unit (infanty, vehicle), so you will have to 
//                     experiment a bit to find the proper explosion. "HE" ammo normally works best.
//                     If you leave this parameter empty (use an empty string: ""), the script will use 
//                     ammo appropriate for the type of triggerclass selected: 
//                       ("Land" or "Man":G_40mm_HE, "LandVehicle":R_Hydra_HE, "Tank":G_40mm_HE)
//
//      "minename"   : Class name of the mine that should be placed to be created. 
//      ("Mine")       The two that are currently available in ArmA are "Mine" and "MineE", 
//                     but you can also use any other objects, or addon elements.
//                     For invisible mines, use "HeliHEmpty" (don't use "Logic" as it takes up more system resources).
//
//      height       : The height the mine should be placed at.
//      (0/.025)       A good height for Western mines ("Mine") is 0.00, and 0.025 for Eastern ones ("MineE").
//
//  While these parameters are optional, there cannot be any "gaps" when assigning them,
//  i.e. you could enter just [triggerclass,quantity,range], but NOT [triggerclass,quantity,explosion].
// 
//  Some examples:
//  nil=["areamarker","LandVehicle",50]... 
//      create 50 mines that are triggered by any vehicles (cars & tanks)
//  nil=["areamarker","Tank",10,5, "Sh_120_HE", "MineE",0.025]... 
//      create 10 Eastern-style anti-tank mines with a triggering radius of 5 meters
//  nil=["areamarker","Land",10,5, "Bo_GBU12_LGB", "RoadCone"]... 
//      create 10 road cones which will detonate a massive bomb if anybody comes within 5 meters
//
// ---------------------------------------------------------------------------------------------------------
//
// By default the marker that's used to designate the area is hidden after the script initialization.
// To keep the marker visible, add the "SHOWMARKER" flag to the parameter list.
//  nil=["areamarker","showmarker"] execVM "minefield.sqf" 
// 
// You can also create warning signs at the corners of the mined area by using the "showsigns" flag.
//  nil=["areamarker","showsigns"] execVM "minefield.sqf" 
//
// ---------------------------------------------------------------------------------------------------------
//
// If you are using the default ArmA mines ("Mine" or "MineE"), they can be defused by units
// in the qualified classes (e.g. Engineers, Special Forces).
//
// Exploding mines can cause nearby other mines to detonate as well, causing a chain reaction.
//
// =========================================================================================================

if (!isNull(server) && !(local server)) exitWith{};
// replace the above check with the following line for ArmA 1.06 and higher
// if !(isServer) exitWith {};

// has the function been initialized yet?
if (isNil("KRON_MF_Tot")) then {
  KRON_MF_Tot=0;
  KRON_MF_Mines=[];
	KRON_MF_Blowup = {private["_near"]; _near=[_this select 0,_this select 1] nearObjects [_this select 3,_this select 2]; {if ((_x select 0) in _near) then {call compile format["KRON_MT_%1_ON=true",_x select 1]; deleteVehicle (_x select 0)}} forEach KRON_MF_Mines};
};

// shared functions
_rotpoint = {private["_cx","_cy","_rx","_ry","_cd","_sd","_ad","_tx","_ty","_xout","_yout"];_cx=_this select 0; _cy=_this select 1; _rx=_this select 2; _ry=_this select 3; _cd=_this select 4; _sd=_this select 5; _ad=_this select 6; _tx=_this select 7; _ty=_this select 8; _xout=if (_ad!=0) then {_cx+ (_cd*_tx - _sd*_ty)} else {_cx+_tx}; _yout=if (_ad!=0) then {_cy+ (_sd*_tx + _cd*_ty)} else {_cy+_ty}; [_xout,_yout,0]};
_makeobj = {private["_o","_p","_o1","_x","_y","_z","_d"]; _o=_this select 0; _p=_this select 1; _x=_p select 0; _y=_p select 1; _z=_p select 2; _d=_this select 2; _o1 = _o createVehicle [_x,_y,_z]; _o1 setPos [_x,_y,_z]; _o1 setDir _d; _o1;};

// read marker definition
_areamarker = _this select 0;
_areadir = (markerDir _areamarker);
// trig values
_cosdir=cos(_areadir*-1);
_sindir=sin(_areadir*-1);
// size 
_areasize = getMarkerSize _areamarker;
_sizeX = _areasize select 0;
_sizeY = _areasize select 1;
// position 
_centerpos = getMarkerPos _areamarker;
_centerX = abs(_centerpos select 0);
_centerY = abs(_centerpos select 1);

// hide marker
_sm=_this find "SHOWMARKER";
if (_sm==-1) then {_sm=_this find "showmarker"};
if (_sm>-1) then {
  _this = _this - [_this select _sm];
} else {  
  _areamarker setMarkerPos [abs(getMarkerPos _areamarker select 0)*-1, abs(getMarkerPos _areamarker select 1)*-1];
};
// show warning signs
_sm=_this find "SHOWSIGNS";
if (_sm==-1) then {_sm=_this find "showsigns"};
if (_sm>-1) then {
  _pos = [_centerX,_centerY,_sizeX,_sizeY,_cosdir,_sindir,_areadir,_sizeX*-1,_sizeY*-1] call _rotpoint;
	["Danger",_pos,_areadir] call _makeobj;
  _pos = [_centerX,_centerY,_sizeX,_sizeY,_cosdir,_sindir,_areadir,_sizeX,_sizeY*-1] call _rotpoint;
	["Danger",_pos,_areadir] call _makeobj;
  _pos = [_centerX,_centerY,_sizeX,_sizeY,_cosdir,_sindir,_areadir,_sizeX*-1,_sizeY] call _rotpoint;
	["Danger",_pos,_areadir] call _makeobj;
  _pos = [_centerX,_centerY,_sizeX,_sizeY,_cosdir,_sindir,_areadir,_sizeX,_sizeY] call _rotpoint;
	["Danger",_pos,_areadir] call _makeobj;
  _this = _this - [_this select _sm];
};

// read optional parameters
_trigact="Land";
if (count _this>1) then {_trigact = _this select 1};
_max=((_sizeX)*(_sizeY))/10; 
if (count _this>2) then {_max = _this select 2};
_trigrange=.75;
if (count _this>3) then {_trigrange = _this select 3};
_trigobj="G_40mm_HE";
if (count _this>4) then {_trigobj = _this select 4};
if (_trigobj=="") then {
  _trigobj="G_40mm_HE";
  if (_trigact=="LandVehicle") then {_trigobj="R_Hydra_HE"};
  if (_trigact=="Tank") then {_trigobj="G_40mm_HE"};
};
_mineobj="MINE"; 
if (count _this>5) then {_mineobj = _this select 5};
_z=if(({_x=="MineE"} count [_mineobj])>0) then {.025} else {0};
if (count _this>6) then {_z = _this select 6};

// are all the object legit?
_chk=[_mineobj,[0,0,0],0] call _makeobj;
if (isNull _chk) exitWith {hintc format["Object class ""%1"" does not exist!",_mineobj]};
deleteVehicle _chk;
_chk=[_trigobj,[0,0,200],0] call _makeobj;
if (isNull _chk) exitWith {hintc format["Ammo class ""%1"" does not exist!",_trigobj]};
deleteVehicle _chk;
if !(isClass (configFile >> "cfgvehicles" >> _trigact)) exitWith {hintc format["Activator class ""%1"" does not exist!",_trigact]};
_armamine=({_x==_mineobj} count ["Mine","MineE"]>0);


// place the weakest class underground, so it's not always deadly
_tz = if (_trigobj=="G_40mm_HE") then {-5.5} else {0};

// create mines
for [{_i=0}, {_i<_max}, {_i=_i+1}] do {
  KRON_MF_Tot=KRON_MF_Tot+1;
  _x=random (_sizeX*2)-_sizeX; 
  _y=random (_sizeY*2)-_sizeY; 
  _pos = [_centerX,_centerY,_sizeX,_sizeY,_cosdir,_sindir,_areadir,_x,_y] call _rotpoint;
  _tx = _pos select 0; 
  _ty = _pos select 1; 
  _obj=[_mineobj,[_tx,_ty,_z],random 360] call _makeobj;
  KRON_MF_Mines=KRON_MF_Mines+[[_obj,KRON_MF_Tot]];
  _t=createTrigger["EmptyDetector",[_tx,_ty]]; 
  _t setTriggerArea [_trigrange,_trigrange,0,true]; 
  _trigside="ANY"; 
  call compile format["_t setTriggerActivation[""%1"",""PRESENT"",false]",_trigside]; 
  call compile format["_t setTriggerStatements[""((this && '%5' countType thislist>0) || !isNil('KRON_MT_%6_ON'))"", ""'%1' createVehicle [%2,%3,%4]; [%2,%3,%7,'%8'] call KRON_MF_Blowup; deleteVehicle KRON_MT_%6; "",""""]", _trigobj,_tx,_ty,_tz,_trigact,KRON_MF_Tot,_trigrange,_mineobj]; 
	_t setVehicleVarName format["KRON_MT_%1",KRON_MF_Tot]; 
  _t call compile format ["KRON_MT_%1=_t",KRON_MF_Tot]; 
};
  
// check for defused or exploded mines
if (isNil("KRON_MineLoop")) then {
  KRON_MineLoop=true;
  while {(count KRON_MF_Mines)>0} do {
    _active=[];
    {
      _m=_x select 0;
      _t=_x select 1;
      if (isNull _m || (_armamine && ((vectorUp _m select 2)!=1))) then {
        call compile format["deleteVehicle KRON_MT_%1",_t];
      } else {
        _active=_active+[_x];
      }
    } forEach KRON_MF_Mines;
    KRON_MF_Mines=+_active;
    sleep 1;
    //player sidechat format["%1",count KRON_MF_Mines]; 
  };
};

